//
/*	FILE		$Header: longer.h,v 1.23 94/07/11 22:37:58 hmgr Exp $*/
//
/*	This file defines the class 'longer'.  This class defines a */
/*	data type and a operators on the data type that can be used*/
/*	for 64-bit integer arithmetic.  This package assumes that the*/
/*	long data type holds exactly 32-bits.  On machines where this*/
/*	is not true this package will not be portable.*/

#ifndef __LONGER_H_
#define __LONGER_H_

#include <codelibs/boolean.h>

#ifndef _GENERIC

#define _ASPLUS		0x00000001
#define _COMPL		0x00000002
#define _MINUS		0x00000004
#define _ASMINUS	0x00000008
#define _XOR		0x00000010
#define _ASXOR		0x00000020
#define _AND		0x00000040
#define _ASAND		0x00000080
#define _OR		0x00000100
#define _ASOR		0x00000200
#define _ASASR		0x00000400
#define _ASLSL		0x00000800
#define _NOT		0x00001000
#define _EQ		0x00002000
#define _NE		0x00004000
#define _LT		0x00008000
#define _ULT		0x00010000
#define _GT		0x00020000
#define _UGT		0x00040000
#define _LE		0x00080000
#define _ULE		0x00100000
#define _GE		0x00200000
#define _UGE		0x00400000
#define _ASMUL		0x00800000
#define _ASDIV		0x01000000
#define _ASMOD		0x02000000

#ifdef __hp9000s300
#define _GENERIC (_COMPL|_XOR|_AND|_OR|_ULT|_UGT|_ULE|_UGE)
#endif
#ifdef __hp9000s800
#define _GENERIC (~(_MINUS))
#endif
#ifndef _GENERIC
#define _GENERIC (~0)
#endif

#endif /* _GENERIC */

#define MAXLONGER longer(0x7FFFFFFF, 0xFFFFFFFF)
#define MINLONGER longer(0x80000000, 0x00000001)

class longer
{
	friend void _divmod64(longer, longer, longer &, longer &);
protected:
	long _msw;
	long _lsw;
public:
	longer() {}
	longer(int a) { _lsw = a; if (a >= 0) _msw = 0; else _msw = -1; }
	longer(long a) { _lsw = a; if (a >= 0) _msw = 0; else _msw = -1; }
	longer(long a, long b) { _lsw = b; _msw = a; }
	longer(double);

	long msw() const { return _msw; }
	long lsw() const { return _lsw; }

	operator long() const { return _lsw; }
	operator int() const { return _lsw; }
	operator double() const;

	longer operator+(longer) const;
	longer operator-(longer) const;

	longer operator*(longer) const;
	longer operator/(longer) const;
	longer operator%(longer) const;

	longer operator^(const longer &) const;
	longer operator&(const longer &) const;
	longer operator|(const longer &) const;

	longer operator<<(int) const;
	longer operator>>(int) const;
	longer lsr(int) const;

	longer operator-() const;

	longer operator~() const;
	CLBoolean operator!() const;

	CLBoolean operator==(longer) const;
	CLBoolean operator!=(longer) const;
	CLBoolean operator<(longer) const;
	CLBoolean operator>(longer) const;
	CLBoolean operator<=(longer) const;
	CLBoolean operator>=(longer) const;
	CLBoolean ult(const longer &) const;
	CLBoolean ugt(const longer &) const;
	CLBoolean ule(const longer &) const;
	CLBoolean uge(const longer &) const;

	longer operator+=(longer);
	longer operator-=(longer);

	longer operator*=(longer);
	longer operator/=(longer);
	longer operator%=(longer);

	longer operator^=(longer);
	longer operator&=(longer);
	longer operator|=(longer);

	longer operator<<=(int);
	longer operator>>=(int);

	longer operator++(int);
	longer operator--(int);
	longer operator++(void);
	longer operator--(void);
};

#if (_GENERIC & _AND)
inline longer
longer::operator&(const longer &l) const
{
	return longer(_msw & l._msw, _lsw & l._lsw);
}
#endif

#if (_GENERIC & _ASPLUS)
inline longer
longer::operator+=(longer l)
{
	return *this = *this + l;
}
#endif

#if (_GENERIC & _COMPL)
inline longer
longer::operator~() const
{
	return longer(~_msw, ~_lsw);
}
#endif

#if (_GENERIC & _MINUS)
inline longer
longer::operator-(longer l) const
{
	return *this + -l;
}
#endif

#if (_GENERIC & _ASMINUS)
inline longer
longer::operator-=(longer l)
{
	return *this = *this - l;
}
#endif

#if (_GENERIC & _XOR)
inline longer
longer::operator^(const longer &l) const
{
	return longer(_msw ^ l._msw, _lsw ^ l._lsw);
}
#endif

#if (_GENERIC & _ASXOR)
inline longer
longer::operator^=(longer l)
{
	return *this = *this ^ l;
}
#endif

#if (_GENERIC & _ASAND)
inline longer
longer::operator&=(longer l)
{
	return *this = *this & l;
}
#endif

#if (_GENERIC & _OR)
inline longer
longer::operator|(const longer &l) const
{
	return longer(_msw | l._msw, _lsw | l._lsw);
}
#endif

#if (_GENERIC & _ASOR)
inline longer
longer::operator|=(longer l)
{
	return *this = *this | l;
}
#endif

#if (_GENERIC & _ASASR)
inline longer
longer::operator>>=(int i)
{
	return *this = *this >> i;
}
#endif

#if (_GENERIC & _ASLSL)
inline longer
longer::operator<<=(int i)
{
	return *this = *this << i;
}
#endif

#if (_GENERIC & _NOT)
inline CLBoolean
longer::operator!() const
{
	return _lsw ? CL_FALSE : _msw ? CL_FALSE : CL_TRUE;
}
#endif

#if (_GENERIC & _EQ)
inline CLBoolean
longer::operator==(longer l) const
{
	return CLBoolean((_msw != l._msw) ? CL_FALSE : (_lsw == l._lsw));
}
#endif

#if (_GENERIC & _NE)
inline CLBoolean
longer::operator!=(longer l) const
{
	return CLBoolean(!(*this == l));
}
#endif

#if (_GENERIC & _LT)
inline CLBoolean
longer::operator<(longer l) const
{
	return CLBoolean((long)_msw < (long)l._msw ? CL_TRUE : ((long)_msw == (long)l._msw && (unsigned long)_lsw < (unsigned long)l._lsw));
}
#endif

#if (_GENERIC & _ULT)
inline CLBoolean
longer::ult(const longer &l) const
{
	return CLBoolean((unsigned long)_msw < (unsigned long)l._msw ? CL_TRUE : ((unsigned long)_msw == l._msw && _lsw < (unsigned long)l._lsw));
}
#endif

#if (_GENERIC & _GT)
inline CLBoolean
longer::operator>(longer l) const
{
	return CLBoolean(l < *this);
}
#endif

#if (_GENERIC & _UGT)
inline CLBoolean
longer::ugt(const longer &l) const
{
	return l.ult(*this);
}
#endif

#if (_GENERIC & _LE)
inline CLBoolean
longer::operator<=(longer l) const
{
	return CLBoolean(*this == l ? CL_TRUE : *this < l);
}
#endif

#if (_GENERIC & _ULE)
inline CLBoolean
longer::ule(const longer &l) const
{
	return l == *this ? CL_TRUE : ult(l);
}
#endif

#if (_GENERIC & _GE)
inline CLBoolean
longer::operator>=(longer l) const
{
	return CLBoolean(l <= *this);
}
#endif

#if (_GENERIC & _UGE)
inline CLBoolean
longer::uge(const longer &l) const
{
	return l == *this ? CL_TRUE : ugt(l);
}
#endif

#if (_GENERIC & _ASMUL)
inline longer
longer::operator*=(longer l)
{
	return *this = *this * l;
}
#endif

#if (_GENERIC & _ASDIV)
inline longer
longer::operator/=(longer l)
{
	return *this = *this / l;
}
#endif

#if (_GENERIC & _ASMOD)
inline longer
longer::operator%=(longer l)
{
	return *this = *this % l;
}
#endif

#undef _GENERIC
#undef _ASPLUS
#undef _COMPL
#undef _MINUS
#undef _ASMINUS
#undef _XOR
#undef _ASXOR
#undef _AND
#undef _ASAND
#undef _OR
#undef _ASOR
#undef _ASASR
#undef _ASLSL
#undef _NOT
#undef _EQ
#undef _NE
#undef _LT
#undef _ULT
#undef _GT
#undef _UGT
#undef _LE
#undef _ULE
#undef _GE
#undef _UGE
#undef _ASMUL
#undef _ASDIV
#undef _ASMOD

extern longer strtolonger(const char *, char **, int);
extern longer atolonger(const char *);
extern const char *longertostr(longer, int);

extern longer pow(longer, longer);
extern longer gcd(longer, longer);
extern longer sqrt(longer);

#endif /* __LONGER_H_ */
