/* $Header: stringx.h,v 1.56 94/04/13 14:04:59 hmgr Exp $ */

#ifndef __STRINGX_H_
#define __STRINGX_H_

#include <stddef.h>
#include <string.h>
#include <codelibs/boolean.h>

#define streq(a,b) (strcmp(a,b) == 0)
#define streqi(a,b) (strcmpi(a,b) == 0)

/* private buffer variables */
extern void *_strsep_privbuf;
extern void *_strsepb_privbuf;
extern void *_strcmbn_privbuf;
extern void *_strcmbnb_privbuf;

#if defined(__cplusplus)
extern "C"
{
#endif
#if defined(__cplusplus) || defined(__STDC__)
    char *strapp(char *s1, const char *s2);
    char *strnapp(char *s1, const char *s2, size_t n);
    const char *strblds(const char *arg, ...);
    const char *strbldf(const char *fmt, ...);
    char *strchg(char *s1, const char *s2, char *ptr, long n);
    int strcmpi(const char *s1, const char *s2);
    int strncmpi(const char *s1, const char *s2, size_t n);
    char *strstrx(char *s1, const char *s2);
    char *strrstrx(char *s1, const char *s2);
    int strwcmp(const char *pattern, const char *str);
    int strwcmpi(const char *pattern, const char *str);
    char *strwpat(const char *pattern);
    char *strend(const char *str);

    unsigned strhash(const char *key);
    unsigned strhashi(const char *key);

    char *strupper(char *str);
    char *strlower(char *str);

    /* C++ & ANSI require this extern, but it isn't
     * present on all systems:
     */
    char *strdup(const char *s1);

#ifdef __cplusplus
    char *strtokx(char *&ptr, const char *sep);
    char **strsepb(const char *str, const char *sep,
	CLBoolean whsp = CL_TRUE, int *num = NULL, void **bufvar = NULL);
    char *strcmbnb(const char *const *vec, const char *sep = " ",
	void **bufvar = NULL);

    int strvlen(const char *const *vec);
    char **strvdup(const char *const *vec, int num = -1);
    void strvfree(const char *const *vec, int num = -1);
#else /* __STDC__ */
    char *strtokx(char **ptr, const char *sep);
    char **strsep(const char *str, const char *sep,
	CLBoolean whsp, int *num);
    char **strsepb(const char *str, const char *sep,
	CLBoolean whsp, int *num, void **bufvar);
    const char *strcmbn(const char *const *vec, const char *sep);
    char *strcmbnb(const char *const *vec, const char *sep, void **bufvar);

    int strvlen(const char *const *vec);
    char **strvdup(const char *const *vec, int num);
    void strvfree(const char *const *vec, int num);
#endif /* __STDC__ */

#ifdef __OBSOLETE
    size_t nl_strlen(const char *str);	/* __OBSOLETE */
    int strcharsize(const char *str);	/* __OBSOLETE */
#endif /* __OBSOLETE */

#else /* C */

extern char  *strapp(), *strnapp();
extern char  *strblds();
extern char  *strbldf();
extern char  *strchg();
extern int    strcmpi(), strncmpi();
extern void   strfree();
extern char  *strstrx(), strrstrx();
extern int    strwcmp(), strwcmpi();
extern char  *strwpat();
extern char  *strend();
extern char  *strtokx();

extern unsigned   strhash();
extern unsigned   strhashi();

extern char      *strupper(), *strlower();

extern char     **strsepb();
extern char      *strcmbnb();

extern char     **strvdup();
extern void       strvfree();
extern int        strvlen();

#ifdef __OBSOLETE
extern int        strcharsize();	/* __OBSOLETE */
extern size_t     nl_strlen();		/* __OBSOLETE */
#endif /* __OBSOLETE */

#endif /* C */
#if defined(__cplusplus)
}

#if defined(apollo)
#include <stdlib.h>
#else
#include <malloc.h>
#endif /* apollo */

inline char *strins(char *s1, const char *s2, char *ptr)
    {return strchg(s1, s2, ptr, 0);}
inline char *strdel(char *s1, char *ptr, long n)
    {return strchg(s1, NULL, ptr, n);}
inline char *strnew(size_t len) { return (char*)malloc(len + 1); }
#if defined(__sun__) || defined(__sun)
inline void  strfree(const char *s)
	{ if (s != NULL) free((char *)s); }
#else
inline void  strfree(const char *s)
	{ if (s != NULL) free((void *)s); }
#endif
#ifdef BSD
inline char *strdup(const char *s)
	{ return strcpy((char*)malloc(strlen(s) + 1), s); }
#endif

/* private buffer funcs - we use inlines to handle default args properly */
inline char **strsep(const char *str, const char *sep,
		     CLBoolean whsp = CL_TRUE, int *num = NULL)
	{ return strsepb(str, sep, whsp, num, &_strsep_privbuf); }
inline const char *strcmbn(const char *const *vec, const char *sep = " ")
	{ return strcmbnb(vec, sep, &_strcmbn_privbuf); }

#define sTr const char *
inline const char *strbld(sTr a1) { return strblds(a1, NULL); }
inline const char *strbld(sTr a1, sTr a2) { return strblds(a1, a2, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3)
	{ return strblds(a1, a2, a3, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4)
	{ return strblds(a1, a2, a3, a4, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5)
	{ return strblds(a1, a2, a3, a4, a5, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6)
	{ return strblds(a1, a2, a3, a4, a5, a6, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11, sTr a12)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11,
		a12, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11, sTr a12, sTr a13)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11,
		a12, a13, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11, sTr a12, sTr a13, sTr a14)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11,
		a12, a13, a14, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11, sTr a12, sTr a13, sTr a14,
	sTr a15)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11,
		a12, a13, a14, a15, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11, sTr a12, sTr a13, sTr a14,
	sTr a15, sTr a16)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11,
		a12, a13, a14, a15, a16, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11, sTr a12, sTr a13, sTr a14,
	sTr a15, sTr a16, sTr a17)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11,
		a12, a13, a14, a15, a16, a17, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11, sTr a12, sTr a13, sTr a14,
	sTr a15, sTr a16, sTr a17, sTr a18)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11,
		a12, a13, a14, a15, a16, a17, a18, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11, sTr a12, sTr a13, sTr a14,
	sTr a15, sTr a16, sTr a17, sTr a18, sTr a19)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11,
		a12, a13, a14, a15, a16, a17, a18, a19, NULL); }
inline const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6,
	sTr a7, sTr a8, sTr a9, sTr a10, sTr a11, sTr a12, sTr a13, sTr a14,
	sTr a15, sTr a16, sTr a17, sTr a18, sTr a19, sTr a20)
	{ return strblds(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11,
		a12, a13, a14, a15, a16, a17, a18, a19, a20, NULL); }
extern "C"
    const char *strbld(sTr a1, sTr a2, sTr a3, sTr a4, sTr a5, sTr a6, sTr a7,
	sTr a8, sTr a9, sTr a10, sTr a11, sTr a12, sTr a13, sTr a14, sTr a15,
	sTr a16, sTr a17, sTr a18, sTr a19, sTr a20, sTr a21, ...);
#undef sTr

#else /* C || __STDC__ */

#define strnew(len) ((char*)malloc((len) + 1))
#define strins(s1, s2, ptr) strchg(s1, s2, ptr, 0);
#define strdel(s1, ptr, n) strchg(s1, NULL, ptr, n);

#define strbld strblds

/* macros for funcs that work on top of privbuf versions */
#define strsep(str, sep, w, ret) strsepb(str, sep, w, ret, &_strsep_privbuf)
#define strcmbn(vec, sep) strcmbnb(vec, sep, &_strcmbn_privbuf)

/* for backward compatibility only - __OBSOLETE */
#ifdef __OBSOLETE
#define strpos strstrx /* __OBSOLETE */
#define strrpos strrstrx /* __OBSOLETE */
#endif /* __OBSOLETE */

#endif /* C || __STDC__ */

#endif /* __STRINGX_H_ */
