/*-------------------------------------------------------------------------
 *
 * PCI Bus Dependent Services 
 *
 *-------------------------------------------------------------------------
 */

#ifndef PCI_H
#define PCI_H

#ifdef _KERNEL_BUILD
#include "../h/_inttypes.h"
#else	/* ! _KERNEL_BUILD */
#include <sys/_inttypes.h>
#endif /* _KERNEL_BUILD */

/*-------------------------------------------------------------------------
 *
 * System Private Structures (exposed for macros and strong type checking)
 * No assumptions on size or names should be made!  Driver useable 
 * definitions follow.
 *
 *                       All fields are reserved!
 *
 *-------------------------------------------------------------------------
 */
typedef void *PHOST_ADAPTER;
typedef uint32_t PCI_CFG_TOKEN;

typedef union {
char *addr; uint32_t hndl;
} PCI_PORT_HNDL;

typedef struct _bus_trans_desc {
    uint32_t read_width;  /* performance access typical width in bytes for reads */
    uint32_t write_width; /* performance access typical width in bytes for writes */
    uint8_t reserved[20];         /* reserved */
} BUS_TRANS_DESC;

#if defined(__STDC__)
/*
 * ANSI C Function Prototypes
 */
typedef void (*RD_CFG_B)(PHOST_ADAPTER, PCI_CFG_TOKEN, int, uint8_t *);
typedef void (*RD_CFG_W)(PHOST_ADAPTER, PCI_CFG_TOKEN, int, uint16_t *);
typedef void (*RD_CFG_D)(PHOST_ADAPTER, PCI_CFG_TOKEN, int, uint32_t *);

typedef void (*WR_CFG_B)(PHOST_ADAPTER, PCI_CFG_TOKEN, int, uint8_t);
typedef void (*WR_CFG_W)(PHOST_ADAPTER, PCI_CFG_TOKEN, int, uint16_t);
typedef void (*WR_CFG_D)(PHOST_ADAPTER, PCI_CFG_TOKEN, int, uint32_t);

typedef void (*RD_IO_B)(PHOST_ADAPTER, PCI_PORT_HNDL, unsigned int, uint8_t *);
typedef void (*RD_IO_W)(PHOST_ADAPTER, PCI_PORT_HNDL, unsigned int, uint16_t *);
typedef void (*RD_IO_D)(PHOST_ADAPTER, PCI_PORT_HNDL, unsigned int, uint32_t *);

typedef void (*WR_IO_B)(PHOST_ADAPTER, PCI_PORT_HNDL, unsigned int, uint8_t);
typedef void (*WR_IO_W)(PHOST_ADAPTER, PCI_PORT_HNDL, unsigned int, uint16_t);
typedef void (*WR_IO_D)(PHOST_ADAPTER, PCI_PORT_HNDL, unsigned int, uint32_t);

typedef void (*RD_B)(PHOST_ADAPTER,  uint8_t *, uint8_t *);
typedef void (*RD_W)(PHOST_ADAPTER, uint16_t *, uint16_t *);
typedef void (*RD_D)(PHOST_ADAPTER, uint32_t *, uint32_t *);
typedef void (*RD_DD)(PHOST_ADAPTER, uint64_t *, uint64_t *);

typedef void (*WR_B)(PHOST_ADAPTER, uint8_t *, uint8_t);
typedef void (*WR_W)(PHOST_ADAPTER, uint16_t *, uint16_t);
typedef void (*WR_D)(PHOST_ADAPTER, uint32_t *, uint32_t);
typedef void (*WR_DD)(PHOST_ADAPTER, uint64_t *, uint64_t);

typedef int (*GET_PORT)(PHOST_ADAPTER, uint32_t, unsigned int, PCI_PORT_HNDL*);
typedef int (*UNMAP_PORT)(PHOST_ADAPTER, uint32_t, unsigned int, PCI_PORT_HNDL);

typedef int (*BUS_TRANS)(PHOST_ADAPTER, PCI_CFG_TOKEN, BUS_TRANS_DESC *);

typedef int (*GET_FRU_INFO)(PHOST_ADAPTER, PCI_CFG_TOKEN, int *);

typedef int (*PCI_ATTACH)(uint32_t, struct isc_table_type*);

extern PCI_ATTACH pci_attach;

#else
/*
 * K & R forward declarations
 */
typedef void (*RD_CFG_B)();
typedef void (*RD_CFG_W)();
typedef void (*RD_CFG_D)();

typedef void (*WR_CFG_B)();
typedef void (*WR_CFG_W)();
typedef void (*WR_CFG_D)();

typedef void (*RD_IO_B)();
typedef void (*RD_IO_W)();
typedef void (*RD_IO_D)();

typedef void (*WR_IO_B)();
typedef void (*WR_IO_W)();
typedef void (*WR_IO_D)();

typedef void (*RD_B)();
typedef void (*RD_W)();
typedef void (*RD_D)();
typedef void (*RD_DD)();

typedef void (*WR_B)();
typedef void (*WR_W)();
typedef void (*WR_D)();
typedef void (*WR_DD)();

typedef int (*GET_PORT)();
typedef int (*UNMAP_PORT)();

typedef int (*BUS_TRANS)();
typedef int (*GET_FRU_INFO)();

extern int (*pci_attach)();

#endif

/*
 * These tables should NEVER be accessed by drivers directly!
 */
typedef struct _pci_bus_dep_func_tbl { 
    RD_CFG_B rd_cfg_b; 
    RD_CFG_W rd_cfg_w; 
    RD_CFG_D rd_cfg_d; 
    WR_CFG_B wr_cfg_b; 
    WR_CFG_W wr_cfg_w;
    WR_CFG_D wr_cfg_d; 
    
    RD_IO_B rd_io_b; 
    RD_IO_W rd_io_w; 
    RD_IO_D rd_io_d;
    WR_IO_B wr_io_b; 
    WR_IO_W wr_io_w; 
    WR_IO_D wr_io_d; 

    RD_B rd_uint8;
    RD_W rd_uint16;
    RD_D rd_uint32;
    RD_DD rd_uint64;
    WR_B wr_uint8;
    WR_W wr_uint16;
    WR_D wr_uint32;
    WR_DD wr_uint64;
    
    GET_PORT get_port;
    UNMAP_PORT unmap_port; 

    GET_FRU_INFO get_fru_info; 
    BUS_TRANS bus_trans; 
    uint32_t reserved[8];
} PCI_BUS_DEP_FUNC_TBL, *PPCI_BUS_DEP_FUNC_TBL; 

typedef struct _pci_bus_info { 
    PHOST_ADAPTER host_adapter; 
    PCI_CFG_TOKEN cfg_tok; 
    PPCI_BUS_DEP_FUNC_TBL func_tbl; 
} PCI_BUS_INFO, *PPCI_BUS_INFO;



/*----------------------------------------------------------------------------
 *
 * Driver Useable Definitions
 *
 *----------------------------------------------------------------------------
 */

/*
 * Configuration space offsets.
 */
#define PCI_CS_VENDOR_ID       0x00
#define PCI_CS_DEVICE_ID       0x02
#define PCI_CS_COMMAND         0x04
#define PCI_CS_STATUS          0x06
#define PCI_CS_REV_ID          0x08
#define PCI_CS_REV_CLASS       0x08
#define PCI_CS_CLASS_BASE      0x0b
#define PCI_CS_CLASS_SUB_CLASS 0x0a
#define PCI_CS_CLASS_PROG_IF   0x09
#define PCI_CS_CACHE_LINE_SIZE 0x0c
#define PCI_CS_LATENCY_TIMER   0x0d
#define PCI_CS_HEADER_TYPE     0x0e
#define PCI_CS_BIST            0x0f

#define PCI_CS_INTERRUPT_PIN   0x3d
#define PCI_CS_INTERRUPT_LINE  0x3c

/*
 * Masks for configuration data
 */
#define MULTI_FUNCTION(header_type) (((header_type) & 0x80) == 0x80)
#define P2P_BRIDGE(header_type) (((header_type) & 0x7f) == 0x01)

/*
 * Bit definitions for configuration space command register
 */
#define PCI_CMD_IO_SPACE          0x001
#define PCI_CMD_MEM_SPACE         0x002
#define PCI_CMD_BUS_MASTER        0x004
#define PCI_CMD_SPEC_CYCLES       0x008
#define PCI_CMD_MEM_WR_INVAL_EN   0x010
#define PCI_CMD_VGA_PAL_SNOOP     0x020
#define PCI_CMD_PARITY_ERR_RESP   0x040
#define PCI_CMD_WAIT_CYCLE_CNTL   0x080
#define PCI_CMD_SERR_ENABLE       0x100
#define PCI_CMD_FAST_BACK_EN      0x200


/*
 * PCI Id
 */
typedef struct _PCI_ID {
    uint16_t vendor_id;
    uint16_t device_id;
} PCI_ID;

/*-------------------------------------------------------------------------
 *
 *
 * PCI Services API
 *
 *
 *-------------------------------------------------------------------------
 */

/*-------------------------------------------------------------------------
 *
 * PCI Configuration Space Access API
 *
 *-------------------------------------------------------------------------
 */

/*
 * pci_read_cfg_uint8_isc(
 *     IN struct isc_table_type isc, 
 *     IN int reg_num, 
 *     OUT uint8_t *byte_read
 *     );
 */
#define pci_read_cfg_uint8_isc(isc, reg_num, byte_read) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->rd_cfg_b( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	((PPCI_BUS_INFO)(isc->bus_info))->cfg_tok, \
	reg_num, \
	byte_read) 

/*
 * pci_read_cfg_uint16_isc(
 *     IN struct isc_table_type isc, 
 *     IN int reg_num, 
 *     OUT uint16_t *word_read
 *     );
 */
#define pci_read_cfg_uint16_isc(isc, reg_num, word_read) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->rd_cfg_w( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	((PPCI_BUS_INFO)(isc->bus_info))->cfg_tok, \
	reg_num, \
	word_read) 

/*
 * pci_read_cfg_uint32_isc(
 *     IN struct isc_table_type isc, 
 *     IN int reg_num, 
 *     OUT uint32_t *dword_read
 *     );
 */
#define pci_read_cfg_uint32_isc(isc, reg_num, dword_read) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->rd_cfg_d( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	((PPCI_BUS_INFO)(isc->bus_info))->cfg_tok, \
	reg_num, \
	dword_read) 


/*
 * pci_write_cfg_uint8_isc(
 *     IN struct isc_table_type isc, 
 *     IN int reg_num, 
 *     IN uint8_t byte_write
 *     );
 */
#define pci_write_cfg_uint8_isc(isc, reg_num, byte_write) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->wr_cfg_b( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	((PPCI_BUS_INFO)(isc->bus_info))->cfg_tok, \
	reg_num, \
	byte_write) 

/*
 * pci_write_cfg_uint16_isc(
 *     IN struct isc_table_type isc, 
 *     IN int re_tg_num, 
 *     IN uint16 word_write
 *     );
 */
#define pci_write_cfg_uint16_isc(isc, reg_num, word_write) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->wr_cfg_w( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	((PPCI_BUS_INFO)(isc->bus_info))->cfg_tok, \
	reg_num, \
	word_write) 

/*
 * pci_write_cfg_uint32_isc(
 *     IN struct isc_table_type isc, 
 *     IN int reg_num, 
 *     IN uint32_t dword_write
 *     );
 */
#define pci_write_cfg_uint32_isc(isc, reg_num, dword_write) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->wr_cfg_d( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	((PPCI_BUS_INFO)(isc->bus_info))->cfg_tok, \
	reg_num, \
	dword_write) 


/*-------------------------------------------------------------------------
 *
 * PCI I/O Port Access API
 *
 *-------------------------------------------------------------------------
 */

/* 
 * Described Above
 *
 * typedef union {
 *   char *addr;
 *   uint32_t hndl;
 * } PCI_PORT_HNDL;
 */

/*
 * pci_read_port_uint8_isc(
 *     IN struct isc_table_type isc, 
 *     IN PCI_PORT_HNDL phndl, 
 *     IN unsigned int offset, 
 *     OUT uint8_t *byte_read
 *     );
 */
#define pci_read_port_uint8_isc(isc, phndl, offset, byte_read) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->rd_io_b( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	phndl, \
	offset, \
	byte_read) 

/*
 * pci_read_port_uint16_isc(
 *     IN struct isc_table_type isc, 
 *     IN PCI_PORT_HNDL phndl, 
 *     IN unsigned int offset, 
 *     OUT uint16_t *word_read
 *     );
 */
#define pci_read_port_uint16_isc(isc, phndl, offset, word_read) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->rd_io_w( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	phndl, \
	offset, \
	word_read) 

/*
 * pci_read_port_uint32_isc(
 *     IN struct isc_table_type isc, 
 *     IN PCI_PORT_HNDL phndl, 
 *     IN unsigned int offset, 
 *     OUT uint32_t *dword_read
 *     );
 */
#define pci_read_port_uint32_isc(isc, phndl, offset, dword_read) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->rd_io_d( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	phndl, \
	offset, \
	dword_read) 

/*
 * pci_write_port_uint8_isc(
 *     IN struct isc_table_type isc, 
 *     IN PCI_PORT_HNDL phndl, 
 *     IN unsigned int offset, 
 *     IN uint8_t byte_write
 *     );
 */
#define pci_write_port_uint8_isc(isc, phndl, offset, byte_write) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->wr_io_b( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	phndl, \
	offset, \
	byte_write) 

/*
 * pci_write_port_uint16_isc(
 *     IN struct isc_table_type isc, 
 *     IN PCI_PORT_HNDL phndl, 
 *     IN unsigned int offset, 
 *     IN uint16_t word_write
 *     );
 */
#define pci_write_port_uint16_isc(isc, phndl, offset, word_write) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->wr_io_w( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	phndl, \
	offset, \
	word_write) 

/*
 * pci_write_port_uint32_isc(
 *     IN struct isc_table_type isc, 
 *     IN PCI_PORT_HNDL phndl, 
 *     IN unsigned int offset, 
 *     IN uint32_t dword_write
 *     );
 */
#define pci_write_port_uint32_isc(isc, phndl, offset, dword_write) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->wr_io_d( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	phndl, \
	offset, \
	dword_write) 

/*
 * pci_get_port_hndl_isc(
 *     IN struct isc_table_type *isc, 
 *     IN uint32_t port_bus_addr, 
 *     IN unsigned int size, 
 *     OUT PCI_PORT_HNDL *phndl
 *     );
 */
#define pci_get_port_hndl_isc(isc, port_bus_addr, size, phndl) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->get_port( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	port_bus_addr, \
	size, \
	phndl)

/*
 * pci_unget_port_hndl_isc(
 *     IN struct isc_table_type *isc, 
 *     IN uint32_t port_bus_addr, 
 *     IN unsigned int size, 
 *     IN PCI_PORT_HNDL phndl
 *     );
 */
#define pci_unget_port_hndl_isc(isc, port_bus_addr, size, phndl) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->unmap_port( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	port_bus_addr, \
	size, \
	phndl)

/*
 * DMA Optimizations
 *
 * A performance driver may want to tune DMA performance by describing the
 * "performance" path DMA transactions to the system.
 *
 */

/*
 * typedef struct _bus_trans_desc {
 *   uint32_t read_width;   
 *   uint32_t write_width;  
 *   uint8_t reserved[20];  
 * } BUS_TRANS_DESC;
 *
 * A width of 0 indicates to use the default.
 */

#define PCI_OKAY 0
#define PCI_BUS_HINTS_NOT_USED 1
#define PCI_BUS_HINTS_BAD_DATA 2

/*
 * int pci_desc_bus_transactions_isc(
 *     IN struct isc_table_type *isc,
 *     IN BUS_TRANS_DESC *desc
 *     );
 */
#define pci_desc_bus_transactions_isc(isc, desc) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->bus_trans( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	((PPCI_BUS_INFO)(isc->bus_info))->cfg_tok, \
	desc)
	
/*
 * Get "field replacable unit" info on a device
 */

/* Possible return codes for the question "What slot is device X in?" */
#define PCI_GET_FRU_INFO_NOT_IMPLEMENTED  0    /* not an available function */
#define PCI_GET_FRU_INFO_DEV_NOT_FOUND    1    /* device not found on any slot */
#define PCI_GET_FRU_INFO_BUILT_IN_FRU     2    /* built-in device with a number */
#define PCI_GET_FRU_INFO_BUILT_IN_NO_FRU  3    /* built-in device with no number */
#define PCI_GET_FRU_INFO_EXPANSION_DEVICE 4    /* expansion device with a slot number */

/*
 * int pci_get_fru_info_isc(
 *     IN struct isc_table_type *isc,
 *     OUT INT *fru_info
 *     );
 */
#define pci_get_fru_info_isc(isc, fru_info) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->get_fru_info( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	((PPCI_BUS_INFO)(isc->bus_info))->cfg_tok, \
	fru_info)

/*
 * Driver "initialization" routines.
 *
 * A driver which desires an init routine should use the following macro or code
 * sequence to associate the function "init_routine" with a particular driver
 * instance. Init routines are special in that they execute after all PCI
 * device/functions are found and all PCI attach routines have had an 
 * opportunity to claim them.
 */
#if defined(__STDC__) || defined(__cplusplus)
    typedef int (*INIT_ROUTINE)(struct isc_table_type *);
#endif
#define CONNECT_INIT_ROUTINE(isc, init_routine) ((isc)->gfsw->init = (init_routine))

/*
 * Attach routine completion macros
 */
#define PCI_ATTACH_DEV_INIT_ERROR(isc) \
    (((struct wsio_if_info *)((isc)->if_info))->flags |= INIT_ERROR)


/* #define PCI_LITTLE_ENDIAN_ONLY */
#ifdef PCI_LITTLE_ENDIAN_ONLY

/*
 * These macros will be used for inline substitution when the driver only
 * handles PCI cards (thus no dynamic check required.)
 */
#define WRITE_REG_UINT8_ISC(isc, addr, value) \
    (*((volatile uint8_t *)(addr)) = (value))

#define WRITE_REG_UINT16_ISC(isc, addr, value) \
    (*((volatile uint16_t *)(addr)) = (((value) >> 8) + (((value) & 0xff) << 8)))

#define WRITE_REG_UINT32_ISC(isc, addr, value) \
    (*((volatile uint32_t *)(addr)) = (uint32_t)(((value) >> 24) \
    + (((value) & 0xff0000) >> 8) + (((value) & 0xff00) << 8) + (((value) & 0xff) << 24)))

#define READ_REG_UINT8_ISC(isc, addr, value) \
    (*((uint8_t *)value) = *((volatile uint8_t *)(addr)))

#define READ_REG_UINT16_ISC(isc, addr, value) \
{ \
    uint16_t val = *((volatile uint16_t *)(addr)); \
        *((uint16_t*)value) = ((val >> 8) + ((val & 0xff) << 8))); \
}

#define READ_REG_UINT32_ISC(isc, addr, value) \
{ \
    uint32_t val = *((volatile uint32_t *)(addr)); \
        *((uint32_t*)value) = (uint32_t)((val >> 24) + ((val & 0x00ff0000) >> 8) \
        + ((val & 0x0000ff00)<< 8) + ((val & 0x000000ff) << 24))); \
}

#else

/*
 * These macros resolve into bus adapter specific calls
 */
#define WRITE_REG_UINT8_ISC(isc, addr, value) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->wr_uint8( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	addr, \
	value)

#define WRITE_REG_UINT16_ISC(isc, addr, value) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->wr_uint16( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	addr, \
	value)

#define WRITE_REG_UINT32_ISC(isc, addr, value) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->wr_uint32( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	addr, \
	value)

#define READ_REG_UINT8_ISC(isc, addr, value) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->rd_uint8( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	addr, \
	value)

#define READ_REG_UINT16_ISC(isc, addr, value) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->rd_uint16( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	addr, \
	value)

#define READ_REG_UINT32_ISC(isc, addr, value) \
    ((PPCI_BUS_INFO)(isc->bus_info))->func_tbl->rd_uint32( \
	((PPCI_BUS_INFO)(isc->bus_info))->host_adapter, \
	addr, \
	value)
#endif


/*
 * DDG1 WSIO Functions and types
 */
#if defined(__STDC__) || defined(__cplusplus)
typedef int (*WSIO_ISR_OLD)(void *, void *);
#endif

/*
 * Driver Debugging Assistance
 */

#ifdef DBG_PCI 
#define DbgPrint(str) DbgPrintf str
#define DLOG DbgPrintf("PCI DEBUG: FILE = \"%s\"  LINE = %d\n", __FILE__, __LINE__)
#else
#define DbgPrint(str) 
#define DLOG
#endif

#endif
