/*
 * @(#)sio_drv.h: $Revision: 1.7.98.3 $ $Date: 95/12/16 01:12:49 $
 * $Locker: BERT $
 */

/****************************************************************************
** Allow multiple inclusion of this header file
****************************************************************************/

#ifndef _SIO_HDR_INCLUDED
#define _SIO_HDR_INCLUDED

/****************************************************************************
** Include files
****************************************************************************/

#ifdef _KERNEL_BUILD
#include "../h/conf.h"              /* for drv_info_t & drv_ops_t */
#include "../h/types.h"             /* for dev_t */
#include "../h/mp.h"                /* for uniprocessor */
#include "../sio/iotree.h"          /* for port_num_type */
#include "../sio/llio.h"
#else   /* ! _KERNEL_BUILD */
#include <sys/conf.h>
#include <sys/types.h>
#include <sys/mp.h>
#include <sio/iotree.h>
#include <sio/llio.h>
#endif /* _KERNEL_BUILD */

#ifndef NULL
		#undef NULL
#define NULL 0
#endif

/****************************************************************************
** The driver header is used in sio_install_driver(SIO2) to describe
** an SIO device driver.  Configuration information previously contained in
** /etc/master is included in these structures.  The driver header must be
** included in every SIO driver.  The driver header is defined by the
** following structures:
******************************************************************************/

typedef struct add_child {          /* add child information */
    char            *name;          /* name of add child driver ("disc3") */
    int             (*install)();   /* ptr to the add child install routine */
} add_child_t;

typedef struct overlay {            /* overlay information */
    char            *top;           /* top name of overlay top ("disc2") */
    char            *base;          /* base name of overlay base ("hpfl0") */
} overlay_t;

typedef struct drv_data {           /* SIO specific information */
    char            *module_name;   /* module name ("disc3") */
    unsigned int    type;           /* type of module: device, interface, bus */
    unsigned int    flag;           /* flag for SIO information */
    unsigned int    mask;           /* mask addressing portions of dev_t */
    char            **parents;      /* ptr to array of parent names */
    add_child_t     *add_child;     /* ptr to array of add child structures */
    overlay_t       *overlay;       /* ptr to the overlay structure */
    int             (*idy)();       /* ptr to identify func ("disc3_idy") */
    int             (*entry)();     /* ptr to the port server entry point */
    int             (*attach)();    /* ptr to the driver attach func */
    int             (*init)();      /* ptr to the driver initialization func */
    int             (*get_minor)(); /* ptr to the driver build minor func */
    int             (*get_port)();  /* ptr to the driver get port num func */
} drv_data_t;

typedef struct sio_info {           /* SIO driver information */
    drv_ops_t       *sio_drv_ops;   /* ptr to struct of driver entry points */
    drv_info_t      *sio_drv_info;  /* ptr to struct of driver information */
    drv_data_t      *sio_drv_data;  /* ptr to struct of SIO specific info */
} sio_info_t;

/****************************************************************************
** SIO flag constants
******************************************************************************/

#define SIO_NO_MGR            0x1   /* hardware without software (target) */
#define SIO_BOUND             0x2   /* at least one bound manager */
#define SIO_INIT              0x4   /* call init routine even if not bound */
#define SIO_LDM               0x8   /* the driver is a LDM */
#define SIO_INITED            0x10  /* driver init routine already called */
#define SIO_INSTR             0x20  /* the driver is instr0 */

/****************************************************************************
** Mask constants
******************************************************************************/

#define IO_AUTOX_MASK         0x00fffe00
#define IO_CENT_MASK          0x00ff0000
#define IO_DISK_MASK          0x00ffff00
#define IO_GPIO_MASK          0x00ff0000
#define IO_INSTR_MASK         0x00ff0000
#define IO_LAN_MASK           0x00ff0000
#define IO_PRINT_MASK         0x00ffff00
#define IO_SCC_MASK           0x00ff0000
#define IO_SERIAL_MASK        0x00ff0000
#define IO_TAPE_MASK          0x00ffff00
#define IO_TTY_MASK           0x00fff000
#define IO_NO_MASK            0x00000000

/****************************************************************************
** Macro gets a token that the kernel instrumentation routines use to 
** generate a handle.  For now (and probably forever) just use a pointer
** to the I/O tree node.
****************************************************************************/
#define sio_get_id( port_num) sio_port_to_node( port_num)

/****************************************************************************
** The following macro reassigns eims and I/O forwarding processors.
****************************************************************************/
#define sio_check_eim_ioforw(IO_eim_reg,Port_num,EIM_control_var,Default_eim) { \
	if (!*EIM_control_var) { \
		int eim_mask; \
		eim_mask=sio_reassign_eim_mask(Port_num,*(u_int *)Default_eim);\
		if (eim_mask) { \
			*(u_int *)Default_eim = *IO_eim_reg = eim_mask; \
			sio_reassign_ioforw(Port_num,*Default_eim); \
			*EIM_control_var = 1; \
		} \
	} \
}
/****************************************************************************
** The following macros reassign eims.
****************************************************************************/
#define sio_check_eim(IO_eim_reg,Port_num,EIM_control_var,Default_eim) { \
	if (!*EIM_control_var) { \
		int eim_mask; \
		eim_mask=sio_reassign_eim_mask(Port_num,*(u_int *)Default_eim);\
		if (eim_mask) { \
			*(u_int *)Default_eim = *IO_eim_reg = eim_mask; \
			*EIM_control_var = 1; \
		} \
	} \
}
#define sio_get_new_eim(Port_num,EIM_control_var,Default_eim) \
        (*EIM_control_var ? 0 : \
                (*EIM_control_var=sio_reassign_eim_mask(Port_num,Default_eim)))

/****************************************************************************
** The following macro schedules I/O forwarding.
** Note the use of the scratch field in struct buf for b_strategy.
****************************************************************************/
#define b_strategy b_s2 
#define sio_sched_strategy(bp,func,ret) { \
	if (!uniprocessor) { \
		bp->b_strategy = (long)func; \
		*ret = sio_ioforw_sched(bp); \
	} \
	else { \
		*ret = func(bp); \
	} \
}

/****************************************************************************
** External declarations
******************************************************************************/

#ifndef _SIO_CDIO_INCLUDED

#ifdef __STDC__

/* ANSI C Function prototypes */

extern int sio_get_hw_path_str
    ( port_num_type port_number, char *hw_path_str);
extern int sio_put_desc
    ( port_num_type port_number, char *desc_str);
extern int sio_scsi_put_desc
    ( port_num_type port_num, char *vendor_id, char *product_id, char *s);
extern int sio_put_pda
    ( port_num_type port_number, void *pda, sio_info_t* header);
extern void *sio_get_pda
    ( dev_t dev, sio_info_t* header);
extern void *sio_port_to_node
    ( port_num_type port_num);
extern int sio_get_iodc
    ( int dev_inst, sio_info_t *header, void *iodc_ptr);
extern int io_port_info
    ( port_num_type port_num, io_port_info_type *info);
extern int sio_port_info
    ( port_num_type port_num, sio_port_info_t *info);
extern int sio_card_inst
    ( port_num_type port_num, int *card_inst);
extern void sio_inq_desc
    ( char *desc, char *inquiry_data);
extern void sio_model_desc
    ( char *desc, bit8 *model_num);
extern void sio_amigoid_desc
    ( char *desc, int amigoid);
extern int sio_replace_desc
    ( port_num_type port_number, char *desc_str);
extern int sio_identify
    ( port_num_type port_number, void **id_ptr, void **desc_ptr);
extern int sio_probe_hw_path
    ( port_num_type port_number, probe_req_type *req);

#else   /* __STDC__ */

/* K&R Function prototypes */
extern int  sio_get_hw_path_str( );
extern int  sio_put_desc( );
extern int  sio_scsi_put_desc( );
extern int  sio_put_pda( );
extern void *sio_get_pda( );
extern void *sio_port_to_node( );
extern int  sio_get_iodc( );
extern int  io_port_info( );
extern int  sio_port_info( );
extern int  sio_card_inst( );
extern void sio_inq_desc( );
extern void sio_model_desc( );
extern void sio_amigoid_desc( );
extern int  sio_replace_desc( );
extern int  sio_identify( );
extern int  sio_probe_hw_path( );

#endif /* __STDC__ */

#endif /* _SIO_CDIO_INCLUDED */

#endif /* _SIO_HDR_INCLUDED */
