/* HPUX_ID: @(#) $Revision: 76.2 $  */
#ifndef _TIME_INCLUDED
#define _TIME_INCLUDED

#include <sys/stdsyms.h>

/*
 * For CASPEC, include tm_* macros and identifiers here
 * But for all other cases, maintain definitions in sys/time.h
 */

#ifdef _INCLUDE_HPUX_SOURCE
#include <sys/time.h>
#else /* _INCLUDE_HPUX_SOURCE */

/* ANSI C time constants, types, and structures */
#  ifdef _INCLUDE__STDC__

#    ifndef CLOCKS_PER_SEC
#      define CLOCKS_PER_SEC 1000000
#    endif

#    ifndef NULL
#      define NULL 0
#    endif

#    ifndef _CLOCK_T
#      define _CLOCK_T
       typedef unsigned long clock_t;
#    endif /* _CLOCK_T */

#    ifndef _TIME_T
#      define _TIME_T
       typedef long time_t;
#    endif /* _TIME_T */

#    ifndef _SIZE_T
#      define _SIZE_T
       typedef unsigned int size_t;
#    endif /* _SIZE_T */

     /* Structure used with gmtime(), localtime(), mktime(), strftime(). */
#ifndef _STRUCT_TM
#	define _STRUCT_TM
     struct tm {
        int tm_sec;	/* second (0-61, allows for leap seconds) */
        int tm_min;	/* minute (0-59) */
        int tm_hour;	/* hour (0-23) */
        int tm_mday;	/* day of the month (1-31) */
        int tm_mon;	/* month (0-11) */
        int tm_year;	/* years since 1900 */
        int tm_wday;	/* day of the week (0-6) */
        int tm_yday;	/* day of the year (0-365) */
        int tm_isdst;	/* non-0 if daylight savings time is in effect */
     };
#endif /* _STRUCT_TM */
#  endif /* _INCLUDE__STDC__ */

/*
 * CLK_TCK is needed by the kernel, and also in the POSIX namespace.
 */

#ifdef _INCLUDE_POSIX_SOURCE
#  ifndef CLK_TCK
#    define CLK_TCK 100
#  endif /* CLK_TCK */
#endif


/* Function prototypes and external variable declarations */

#ifndef _KERNEL
#ifdef __cplusplus
   extern "C" {
#endif /* __cplusplus */

#ifdef _INCLUDE__STDC__
#  ifdef _PROTOTYPES
#    ifndef _DIFFTIME
#      define _DIFFTIME
       extern double difftime(time_t, time_t);
#    endif /** _DIFFTIME **/
#    ifndef _MKTIME
#      define _MKTIME
       extern time_t mktime(struct tm *);
#    endif /** _MKTIME **/
#    ifndef _TIME
#      define _TIME
       extern time_t time(time_t *);
#    endif
#    ifndef _ASCTIME
#      define _ASCTIME
       extern char *asctime(const struct tm *);
#    endif
#    ifndef _CTIME
#      define _CTIME
       extern char *ctime(const time_t *);
#    endif
#    ifndef _GMTIME
#      define _GMTIME
       extern struct tm *gmtime(const time_t *);
#    endif
#    ifndef _LOCALTIME
#      define _LOCALTIME
       extern struct tm *localtime(const time_t *);
#    endif
#    ifndef _STRFTIME
#      define _STRFTIME
       extern size_t strftime(char *, size_t, const char *, const struct tm *);
#    endif
#      ifdef _REENTRANT
#        ifndef _ASCTIME_R
#          define _ASCTIME_R
           extern int asctime_r(const struct tm *, char *, int);
#        endif
#        ifndef _GMTIME_R
#	        define _GMTIME_R
           extern int gmtime_r(const time_t *, struct tm *);
#         endif
#         ifndef _LOCALTIME_R
#           define _LOCALTIME_R
            extern int localtime_r(const time_t *, struct tm *);
#         endif
#       endif
#  else /* not _PROTOTYPES */

#    ifndef _DIFFTIME
#      define _DIFFTIME
       extern double difftime();
#    endif

#    ifndef _MKTIME
#      define _MKTIME
       extern time_t mktime();
#    endif

#    ifndef _TIME
#      define _TIME
       extern time_t time();
#    endif

#    ifndef _ASCTIME
#      define _ASCTIME
       extern char *asctime();
#    endif

#    ifndef _CTIME
#      define _CTIME
       extern char *ctime();
#    endif

#    ifndef _GMTIME
#      define _GMTIME
       extern struct tm *gmtime();
#    endif

#    ifndef _LOCALTIME
#      define _LOCALTIME
       extern struct tm *localtime();
#    endif

#    ifndef _STRFTIME
#      define _STRFTIME
       extern size_t strftime();
#    endif

#    ifdef _REENTRANT

#      ifndef _ASCTIME_R
#        define _ASCTIME_R
         extern int asctime_r();
#      endif

#      ifndef _GMTIME_R
#        define _GMTIMRE_R
         extern int gmtime_r();
#      endif

#      ifndef _LOCALTIME_R
#        define _LOCALTIME_R
         extern int localtime_r();
#      endif

#    endif
#  endif /* not _PROTOTYPES */

#  ifdef _CLASSIC_ANSI_TYPES

#    ifndef _CLOCK
#      define _CLOCK
       extern long clock();
#    endif

#  else /* not _CLASSIC_ANSI_TYPES */
#    ifdef _PROTOTYPES
#      ifndef _CLOCK
#        define _CLOCK
         extern clock_t clock(void);
#      endif
#    else /* not _PROTOTYPES */
#      ifndef _CLOCK
#        define _CLOCK
         extern clock_t clock();
#      endif
#    endif /* not _PROTOTYPES */
#  endif /* not _CLASSIC_ANSI_TYPES */
#endif /* _INCLUDE__STDC__ */

#ifdef _INCLUDE_POSIX_SOURCE
#  ifdef _PROTOTYPES

#    ifndef _TZSET
#      define _TZSET
       extern void tzset(void);
#    endif

#  else /* not _PROTOTYPES */

#    ifndef _TZSET
#      define _TZSET
       extern void tzset();
#    endif

#  endif /* not _PROTOTYPES */

#  ifndef _TZNAME
#    define _TZNAME
     extern char *tzname[2];
#  endif

#endif /* _INCLUDE_POSIX_SOURCE */

#ifdef _INCLUDE_XOPEN_SOURCE
#  ifdef _PROTOTYPES

#    ifndef _STRPTIME
#	    define _STRPTIME
       extern char *strptime(const char *, const char *, struct tm *);
#    endif

#  else /* not _PROTOTYPES */

#    ifndef _STRPTIME
#      define _STRPTIME
       extern char *strptime();
#    endif

#  endif /* not _PROTOTYPES */

#  ifndef _TIMEZONE
#    define _TIMEZONE
     extern long timezone;
#  endif

#  ifndef _DAYLIGHT
#    define _DAYLIGHT
     extern int daylight;
#  endif

#endif /* _INCLUDE_XOPEN_SOURCE */

#ifdef _INCLUDE_XOPEN_SOURCE_EXTENDED
#  ifdef _PROTOTYPES

#    ifndef _GETDATE
#      define _GETDATE
       extern struct tm *getdate(const char *string);
#    endif

#  else /* !_PROTOTYPES */

#    ifndef _GETDATE
#      define _GETDATE
       extern struct tm *getdate();
#    endif

#  endif /* !_PROTOTYPES */

#    ifndef _GETDATE_ERR
#      define _GETDATE_ERR
       extern int getdate_err;
#    endif

#endif /* _INCLUDE_XOPEN_SOURCE_EXTENDED */

#ifdef _INCLUDE_HPUX_SOURCE
#  ifdef _PROTOTYPES

#    ifndef _GETDATE_R
#      define _GETDATE_R
       extern int getdate_r(const char *, struct tm *, int *);
#    endif

#  else /* !_PROTOTYPES */

#    ifndef _GETDATE_R
#      define _GETDATE_R
       extern int getdate_r();
#    endif

#  endif /* !_PROTOTYPES */
#endif /* _INCLUDE_HPUX_SOURCE */

#ifdef __cplusplus
   }
#endif /* __cplusplus */
#endif /* not _KERNEL */

# endif /*  _INCLUDE_HPUX_SOURCE */

#endif /* _TIME_INCLUDED */
